/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx.topology;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Container.State;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.admin.topology.Subdomain;
import org.ow2.petals.clientserverapi.topology.TopologyService;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyValues;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class TopologyUtils {

    private static final Map<String, PortType> PORT_TYPE_CONVERTER = new HashMap<String, Container.PortType>();

    private static final Map<String, State> STATE_VALUE_CONVERTER = new HashMap<String, Container.State>();
    
    static {
        PORT_TYPE_CONVERTER.put(TopologyService.ContainerPropertyName.CONF_JMX_RMI_PORT, PortType.JMX);
        PORT_TYPE_CONVERTER.put(TopologyService.ContainerPropertyName.CONF_TRANSPORT_TCP_PORT, PortType.TCP_TRANSPORT);

        STATE_VALUE_CONVERTER.put(ContainerPropertyValues.STATE_STARTED, State.STARTED);
        STATE_VALUE_CONVERTER.put(ContainerPropertyValues.STATE_STOPPED, State.STOPPED);
    }   
    
    /**
     * <p>
     * Create a {@link Domain} according to:
     * <ul>
     * <li>the provided topology,</li>
     * <li>a regexp filtering container to add.</li>
     * </ul>
     * </p>
     * <p>
     * If after filtering, a sub-domain has no container, it is not included in the returned topology
     * </p>
     * 
     * @param containerFilterRegexp
     *            A container name regexp to filter the containers to return
     * @return A complete domain definition or limited to the provided filters.
     */

    public static final Domain createListOfDomains(final Set<Map<String, String>> topology,
            final String containerFilterRegexp) {
        Domain domain = null;
        final List<Subdomain> subdomains = new ArrayList<Subdomain>();
        final Map<String, List<Container>> containers = new HashMap<String, List<Container>>();
        
        final Pattern containerNamePattern = Pattern.compile(containerFilterRegexp);

        for (final Map<String, String> map : topology) {
            final String topologyType = map.get(TopologyService.TopologyItem.TOPOLOGY_TYPE);
            
            if (topologyType.equals(TopologyService.TopologyItem.DOMAIN)) {
                final String name = map.get(TopologyService.DomainPropertyName.CONF_DOMAIN_NAME);
                domain = new Domain(name);
            } else if (topologyType.equals(TopologyService.TopologyItem.SUBDOMAIN)) {
                final String name = map.get(TopologyService.SubdomainPropertyName.CONF_SUBDOMAIN_NAME);
                final Subdomain subdomain = new Subdomain(name);
                subdomains.add(subdomain);
            } else if (topologyType.equals(TopologyService.TopologyItem.CONTAINER)) {
                final String subdomain = map.get(TopologyService.ContainerPropertyName.CONF_SUBDOMAIN_NAME);
                
                final List<Container> containersForThisSubDomain;
                if(containers.containsKey(subdomain)) {
                    containersForThisSubDomain = containers.get(subdomain);
                } else {
                    containersForThisSubDomain = new ArrayList<Container>();
                    containers.put(subdomain, containersForThisSubDomain);
                }
                
                final String name = map.get(TopologyService.ContainerPropertyName.CONF_CONTAINER_NAME);
                final Matcher containerNameMatcher = containerNamePattern.matcher(name);
                if (containerNameMatcher.matches()) {

                    final String host = map.get(TopologyService.ContainerPropertyName.CONF_HOST);
                    final String jmxUsername = map.get(TopologyService.ContainerPropertyName.CONF_USER);
                    final String jmxPassword = map.get(TopologyService.ContainerPropertyName.CONF_PWD);
                    final String state = map.get(TopologyService.ContainerPropertyName.CONF_STATE);

                    final HashMap<PortType, Integer> ports = new HashMap<PortType, Integer>();
                    for (final String portType : PORT_TYPE_CONVERTER.keySet()) {
                        if (map.containsKey(portType)) {
                            final int portValue = Integer.valueOf(map.get(portType));
                            ports.put(PORT_TYPE_CONVERTER.get(portType), portValue);
                        }
                    }

                    final Container container = new Container(name, host, ports, jmxUsername, jmxPassword,
                            STATE_VALUE_CONVERTER.get(state) == null ? State.UNKNOWN : STATE_VALUE_CONVERTER.get(state));
                    containersForThisSubDomain.add(container);
                }
            } else {
                final String errorMessage = String.format("Unrecognized topology type: '%s'", topologyType);
                throw new UncheckedException(errorMessage);
            }
        }
        
        if(domain == null) {
            String errorMessage = "Error in the topology map: no domain";
            throw new UncheckedException(errorMessage);
        }
        
        final Iterator<Subdomain> itSubdomains = subdomains.iterator();
        while (itSubdomains.hasNext()) {
            final Subdomain subdomain = itSubdomains.next();
            final List<Container> containersOfSubdomain = containers.get(subdomain.getName());
            if (containersOfSubdomain.size() > 0) {
                subdomain.addContainers(containersOfSubdomain);
            } else {
                // The sub-domain has no container, we removed it from the returned topology
                itSubdomains.remove();
            }
        }    
        domain.addSubdomain(subdomains);
        
        return domain;
    }
    
}
